"""
Examples for GridFIA Spatial Forest Analysis

This package contains example scripts and notebooks demonstrating
how to use GridFIA for various forest analysis tasks.

Part of the FIA Python Ecosystem:
- PyFIA: Survey/plot data analysis (https://github.com/mihiarc/pyfia)
- GridFIA: Spatial raster analysis (this package)
- PyFVS: Growth/yield simulation (https://github.com/mihiarc/pyfvs)
- AskFIA: AI conversational interface (https://github.com/mihiarc/askfia)
""" 